package fxmaju;

import fi.jyu.mit.fxgui.ComboBoxChooser;
import fi.jyu.mit.fxgui.ModalController;
import fi.jyu.mit.fxgui.ModalControllerInterface;
import fi.jyu.mit.fxgui.Dialogs;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.TextField;
import javafx.stage.Stage;
import majuKomponentit.Maju;
import majuKomponentit.Peli;
import majuKomponentit.SailoException;

import java.net.URL;
import java.util.ResourceBundle;

/**
 * @author maurihase
 * @version 12.2.2025
 *
 */
public class majuLisaaPeliController implements ModalControllerInterface<Peli>, Initializable {
 @FXML private Button bLisaa;
 @FXML private Button bPeru;
 @FXML private Label labelVirhe;

 @FXML private ComboBoxChooser<String> PelinLisaosa;
 private final String[] kyllaEi = new String[]{"Kyllä", "Ei"};
 @FXML private ComboBoxChooser<String> arvosanaIGN;
 @FXML private ComboBoxChooser<String> arvosanaMAJU;
 private final String[] arvosanaAsteikko = new String[]{"NR", "1/10", "2/10", "3/10", "4/10", "5/10", "6/10", "7/10", "8/10", "9/10", "10/10"};
 @FXML private ComboBoxChooser<String> pelilleGenre;
 private final String[] genret = new String[]{"Toiminta", "Strategia", "Kauhu", "Rento", "Roolipeli", "Rakentaminen", "Tasohyppely", "Monipeli", "Yhteistyöpeli", "Massiivinen monipeli", "Tappelupeli"};
 @FXML private TextField pelinAlusta;
 @FXML private TextField pelinJulkaisija;
 @FXML private TextField pelinJulkaisu;
 @FXML private TextField pelinNimi;
 @FXML private ComboBoxChooser<String> pelinSuositus;
 @FXML private TextField pelinValmistaja;


 @FXML
 void handleLisaa() {
  uusiPeli();
  tallenna();
  sulje();
 }

 @FXML
 void handlePeru() {
  sulje();
 }

 //—————————————————————————————————————————————————————————————————————————————————————————————————————————————————————
//Atribuutteja
 private Maju maju;
 private Peli peliKohdalla;


 /**
  * avaa tämän ikkunan
  * @param modalityStage ikkuna
  * @param peliKohdalla peli
  * @param maju meidän pääikkuna ja -luokka
  * @return avautunut ikkuna
  */
 public static Peli kysyPeli(Stage modalityStage, Peli peliKohdalla, Maju maju) {
  return ModalController.<Peli, majuLisaaPeliController>showModal(
          majuLisaaPeliController.class.getResource("majuLisaaPeliView.fxml"),
          "Maju",
          modalityStage, peliKohdalla, ctrl -> ctrl.setMaju(maju)
  );
 }


 /**
  * Asettaa majun
  * @param maju pääluokka
  */
 public void setMaju(Maju maju){
  this.maju = maju;
 }


 /**
  * Hakee pelin
  * @return peli
  */
 @Override
 public Peli getResult() {
  return peliKohdalla;
 }


 /**
  * Ehkä pois
  * @param peli niin...
  */
 @Override
 public void setDefault(Peli peli) {
//
 }


 /**
  * Näyttää jotain...
  */
 @Override
 public void handleShown() {
//
 }


 /**
  * Sulkee ikkunan
  * @return sulun
  */
 public boolean sulje() {
  labelVirhe.setText("Kiinni");
  ModalController.closeStage(labelVirhe);
  return true;
 }


 /**
  * Tallennus
  * @return ei mitään, jos onnistuu, muulloin virheilmoitus
  */
 private String tallenna() {
  try {
   maju.tallenna();
   return null;
  } catch (SailoException e) {
   Dialogs.showMessageDialog("Tallennuksessa tuli ongelma!! " + e.getMessage());
   return e.getMessage();
  }
 }


 /**
  * Alustusta
  * @param url nettisivu?
  * @param resourceBundle resurssipaketti
  */
 @Override
 public void initialize(URL url, ResourceBundle resourceBundle) {
  luoValitsimet();
 }


 /**
  * Luo Combobox:lle valitsimet
  */
 private void luoValitsimet() {
  //Ensin genret
  for (int i = 0; i < genret.length; i++) {
   pelilleGenre.add(genret[i]);
  }
  //Seuraavaksi arvosanat
  for (int i = 0; i < arvosanaAsteikko.length; i++) {
   arvosanaMAJU.add(arvosanaAsteikko[i]);
   arvosanaIGN.add(arvosanaAsteikko[i]);
  }
  //Viimeisenä kyllä/ei vastaukset
  for (int i = 0; i < kyllaEi.length; i++) {
   pelinSuositus.add(kyllaEi[i]);
   PelinLisaosa.add(kyllaEi[i]);
  }
 }


 /**
  * Lisää uuden pelin
  */
 protected void uusiPeli() {
  Peli uusi = new Peli(getPeliNimi(), getJulkaisija(), getValmistaja(), getJulkaisuVuosi(), getAlusta(), getLisaosat(), getArvosanaIGN(), getArvosanaMAJU(), getSuositus());
  uusi.rekistoroi();
  maju.asetaPelilleGenret(asetaGenre(), uusi.getTunnusnro());
  peliKohdalla = uusi;
  try{
   maju.lisaa(uusi);
  }catch (SailoException e){
   Dialogs.showMessageDialog("Ongelmia uuden pelin luomisessa! " + e.getMessage());
   return;
  }
 }


 /**
  * Hakee pelin alustan
  * @return tekstinä pelin alustan, jonka käyttäjä on antanut
  */
 public String getAlusta() {
  return pelinAlusta.getText();
 }


 /**
  * Hakee pelin julkaisuvuoden
  * @return käyttäjän antaman julkaisuvuoden pelille
  */
 public int getJulkaisuVuosi() {
  return Integer.parseInt(pelinJulkaisu.getText());
 }


 /**
  * Hakee tiedon pelin valmistajasta
  * @return pelin valmistajan, jonka käyttäjä on antanut
  */
 public String getValmistaja() {
  return pelinValmistaja.getText();
 }


 /**
  * Hakee pelin julkaisijan
  * @return pelin julkaisijan, jonka käyttäjä on antanut pelille
  */
 public String getJulkaisija() {
  return pelinJulkaisija.getText();
 }


 /**
  * Hakee pelin nimen
  * @return pelin nimi, jonka käyttäjä on antanut
  */
 public String getPeliNimi() {
  return pelinNimi.getText();
 }


 /**
  * Katsoo käyttäjän vastauksen suosituksesta
  * @return kyllä/ei
  */
 public String getSuositus() {
  return String.valueOf(pelinSuositus.getValue());
 }


 /**
  * Katsoo käyttäjän vastauksen pelin lisäosista
  * @return kyllä/ei
  */
 public String getLisaosat(){
  return String.valueOf(PelinLisaosa.getValue());
 }


 /**
  * Katsoo käyttäjän asettaman IGN arvosanan
  * @return IGN/10
  */
 public String getArvosanaIGN() {
  return String.valueOf(arvosanaIGN.getValue());
 }


 /**
  * Katsoo käyttäjän asettaman MAJU arvosanan
  * @return MAJU/10
  */
 public String getArvosanaMAJU() {
  return String.valueOf(arvosanaMAJU.getValue());
 }


 public int asetaGenre() {
  return pelilleGenre.getSelectedIndex()+1;
 }
}