package majuKomponentit;

import java.io.*;
import java.util.ArrayList;
import java.util.List;

/**
 * @author jpaaj
 * @version 09.4.2025
 *
 */
public class Genret {
    //attribuutteja
    private static int maksimiGenret = 11;
    private static int lkm = 0;
    private String tiedostoNimi = "";
    private static Genre[] alkiot = new Genre[maksimiGenret];
    private final String tiedostonNimi = "genreNimet.dat";
    private String tiedostonPerusNimi = "nimet";

    /**
     * Pääohjelma
     * @param args ee
     */
    public static void main (String[] args) {

        Genret genret = new Genret();

        Genre tasohyppely = new Genre("Tasohyppely");
        tasohyppely.rekistoroi();

        Genre rento = new Genre("Rento");
        rento.rekistoroi();

        try{
            Genret.lisaa(tasohyppely);
            Genret.lisaa(rento);

            for (int i = 0; i < genret.getLkm(); i++) {
                Genre genre = genret.anna(i);
                System.out.println("Genre nro: " + i);
                genre.tulosta(System.out);
            }
        } catch (SailoException e) {
            System.out.println(e.getMessage());
        }
    }


    /**
     * Lisää genren genrejen taulukkoon
     * @param genre lisättävä genre
     * @throws SailoException virhe ilmoitus täydestä taulusta
     */
    public static void lisaa(Genre genre) throws SailoException{
        if (lkm >= alkiot.length) throw new SailoException("Liikaa genrejä olemassa");
        alkiot[lkm] = genre;
        lkm++;
    }


    /**
     * Palauttaa viitteen i:teen genreen
     * @param i halutun genren paikka
     * @return haluttu genre
     * @throws IndexOutOfBoundsException jos luku on taulun pituuden ulkopuolella
     */
    public Genre anna(int i) throws IndexOutOfBoundsException {
        if (i < 0 || lkm <= i) throw new IndexOutOfBoundsException("Taulun ulkopuolella on " + i);
        return alkiot[i-1];
    }


    /**
     * Lukee haluttua tiedostoa
     * @throws SailoException kun ei osata vieläkään lukea
     */
    public void lueTiedosto() throws SailoException {
        try(BufferedReader fi = new BufferedReader(new FileReader(tiedostonNimi))) {
            String rivi = fi.readLine();
            if (rivi == null) throw new SailoException("Laiton indeksi: " + tiedostonNimi);
            while ((rivi = fi.readLine()) != null) {
                rivi = rivi.trim();
                if (rivi.isEmpty() || rivi.charAt(0) == ';') continue;
                Genre genre = new Genre();
                genre.parse(rivi);
                lisaa(genre);
            }
        }catch (FileNotFoundException fe) {
            throw new SailoException("Tiedoston " + tiedostonNimi + " ei jostain syystä toimi: " + fe.getMessage());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


    /**
     * Tallennetaan tiedosto
     * @throws SailoException kun ei vieläkään osata tallettaa
     */
    public void tallenna() throws SailoException {
        throw new SailoException("En osaa tallettaa " + tiedostoNimi);
    }


    /**
     * Genrejen lukumäärä
     * @return genrejen määrä
     */
    public int getLkm(){
        return lkm;
    }
    
    //testiä
    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonPerusNimi() {
        return tiedostonPerusNimi;
    }


    /**
     * Asettaa tiedoston perusnimen ilan tarkenninta
     * @param nimi tallennustiedoston perusnimi
     */
    public void setTiedostonPerusNimi(String nimi) {
        tiedostonPerusNimi = nimi;
    }


    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonNimi() {
        return getTiedostonPerusNimi() + ".dat";
    }

    
    //testiä päättyy
    
    /**
     * @param loydetyt genret
     * @return palauttaa löydetyt genret
     */
    public List<Genre> annaGenret(List<PelinGenret> loydetyt) {
        List<Genre> genret = new ArrayList<>();
        for (Genre genre : alkiot) {
            for (PelinGenret pelinGenret : loydetyt) {
                if(pelinGenret.getGenreId() == genre.getTunnusnro())  genret.add(genre);
            }
        }
        return genret;
    }

    public void tulosta(PrintStream os) {
        //
    }
}
