package majuKomponentit;

import java.io.File;
import java.util.List;

/**
 * @author jpaaj
 * @version 09.4.2025
 *
 */
public class Maju {
    //Atribuutteja
    private final Pelit pelit = new Pelit();
    private final Genret genret = new Genret();
    private final PelienGenret pelienGenret = new PelienGenret();



    /**
     * Palauttaa pelien määrän
     * @return pelien määrän
     */
    public int getPeleja(){
        return pelit.getLkm();
    }


    /**
     * Palauttaa genrejen määrän
     * @return genrejen määrä
     */
    public int getGenreja(){
        return genret.getLkm();
    }


    /**
     * asettaa pelille genret niiden ID:den perusteella
     * @param genreID genre ID
     * @param peliID peli ID
     */
    public void asetaPelilleGenret(int genreID, int peliID){
        pelienGenret.lisaa(genreID, peliID);
        genret.anna(genreID);
    }


    /**
     * Palauttaa pelien genrejen määrän
     * @return pelin genrejen määrä
     */
    public int getPelienGenret(){
        return pelienGenret.getLkm();
    }


    /**
     * Poistotilanteessa poistaa tunnusnumeroon yhdistyneet tiedot
     * @param numero ,jonka avulla poistetaan yhdistettyä tietoa
     * @return nolla, koska vielä kesken
     */
    @SuppressWarnings("unused")
    public int poista(int numero){
        return 0;
    }


    /**
     * Lisää MAJU:n uuden pelin
     * @param peli lisättävä peli
     * @throws SailoException ,koska vielä ei ole koulutettu
     */
    public void lisaa(Peli peli) throws SailoException {
        pelit.lisaa(peli);
    }


    /**
     * Lisään MAJU:n uuden Pelin Genret, eli yhdistää pelille genrejä
     * @param pelinGenret kyllä
     */
    public void lisaa(PelinGenret pelinGenret){
        pelienGenret.lisaa(pelinGenret);
        genret.anna(pelinGenret.getLkm());
    }


    /**
     * Näyttää i:n pelin
     * @param i monesko peli palautetaan
     * @return viite i:teen peliin
     * @throws IndexOutOfBoundsException jos i on väärin
     */
    public Peli annaPeli(int i) throws IndexOutOfBoundsException {
        return pelit.anna(i);
    }


    /**
     * @param peli pelin ID
     * @return palauttaa pelille määrätyt genret
     */
    public List<Genre> annaGenret(Peli peli) {
        List<PelinGenret> loydetyt = annaPelienGenret(peli);
        return genret.annaGenret(loydetyt);
    }


    public void annaGenre(int genreId) {
        genret.anna(genreId);
    }


    /**
     * lukee pelin tiedot tiedostosta
     * @param peli nimi jota käytetään lukemisessa
     * @return palauttaa pelin genret
     */
    public List<PelinGenret> annaPelienGenret(Peli peli){
        return pelienGenret.annaPelienGenret(peli.getTunnusnro());
    }


    //tesit
    /**
     * Asettaa tiedostojen perusnimet
     * @param nimi uusi nimi
     */
    public void setTiedosto(String nimi) {
        File dir = new File(nimi);
        dir.mkdirs();
        String hakemistonNimi = "";
        if ( !nimi.isEmpty() ) hakemistonNimi = nimi +"/";
        pelit.setTiedostonPerusNimi(hakemistonNimi + "nimet");
        genret.setTiedostonPerusNimi(hakemistonNimi + "genret");
    }
    //testit


    /**
     * Lukee kerhon tiedot teidostosta
     * @throws SailoException jos i väärin
     */
    public void lueTiedostosta() throws SailoException {
        pelit.lueTiedostosta();
        pelienGenret.lueTiedostosta("peliengenret");
        genret.lueTiedosto();
    }


    /**
     * Tallettaa kerhon tiedot teidostoon
     * @throws SailoException jos tallentamisessa on ongelmia
     */
    public void tallenna() throws SailoException {
        pelit.tallenna();
        pelienGenret.tallenna();
    }


    /**
     * Pääohjelma
     * @param args ei käytössä
     */
    public static void main(String[] args) {
        Maju maju = new Maju();

        try {
            Peli vankilapako = new Peli();
            vankilapako.rekistoroi();
            maju.lisaa(vankilapako);
            int id = vankilapako.getTunnusnro();
            PelinGenret tasohyppely = new PelinGenret(id);
            maju.lisaa(tasohyppely);
            PelinGenret renton = new PelinGenret(id);
            maju.lisaa(renton);

            for (int i = 0; i < maju.getPeleja(); i++) {
                Peli peli = maju.annaPeli(i);
                System.out.println("Peli paikassa: " + i);
                peli.tulosta(System.out);
                List<PelinGenret> loytyneet = maju.annaPelienGenret(peli);
                for (PelinGenret genre : loytyneet) {
                    genre.tulosta(System.out);
                }
            }
        } catch (SailoException e) {
            System.out.println(e.getMessage());
        }
    }
}
