package majuKomponentit;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.*;

/**
 * Pelien genret, joka osaa mm lisätä uuden genren
 * 
 * @author jpaaj
 * @version 7.3.2025
 *
 */
public class PelienGenret implements Iterable <PelinGenret>{
    private static String          tiedostonNimi="peliengenret.dat";

    private boolean muutettu = false;
    private String tiedostonPerusNimi = "";

    
    /*taulukko genreistä*/
    private static final Collection<PelinGenret> alkiot = new ArrayList<PelinGenret>();


    /**
     * Genrejen alustaminen
     */
    public PelienGenret() {
        //ei viel mihtää! HOJO!
    }


    /**
     * Lisätään käyttäjän lisäämät genret
     * @param genreID Genren ID
     * @param peliID Pelin ID
     */
    public void lisaa(int genreID, int peliID) {
        PelinGenret genre = new PelinGenret(genreID, peliID);
        alkiot.add(genre);
        muutettu = true;
    }

    /**
     * Lisää uuden genren tietorakenteeseen. 
     * @param genre lisättävä genre
     */
    public void lisaa(PelinGenret genre) {
        alkiot.add(genre);
        muutettu = true;
    }
    
    
    /**
     * Luetaan aikaisemmin annetun nimisestä tiedostosta
     * @param tied tiedosto
     * @throws SailoException jos tulee poikkeus
     */
    public void lueTiedostosta(String tied) throws SailoException {
        setTiedostonPerusNimi(tied);
        try ( BufferedReader fi = new BufferedReader(new FileReader(getTiedostonNimi())) ) {

            String rivi;
            while ( (rivi = fi.readLine()) != null ) {
                rivi = rivi.trim();
                if ( "".equals(rivi) || rivi.charAt(0) == ';' ) continue;
                PelinGenret genre = new PelinGenret();
                genre.parse(rivi); // voisi olla virhekäsittely
                lisaa(genre);
            }
            muutettu = false;

        } catch ( FileNotFoundException e ) {
            throw new SailoException("Tiedosto " + getTiedostonNimi() + " ei aukea");
        } catch ( IOException e ) {
            throw new SailoException("Ongelmia tiedoston kanssa: " + e.getMessage());
        }
    }



    /**
     * Tallennus
     * @throws SailoException vika ilmotus
     */
    public void tallenna() throws SailoException {
        if ( !muutettu ) return;
        
        File fbak = new File (getBakNimi());
        File ftied = new File (getTiedostonNimi());
        fbak.delete();
        ftied.renameTo(fbak);
        
        try ( PrintWriter fo = new PrintWriter(new FileWriter(ftied.getCanonicalPath())) ) {
            for (PelinGenret genre : this) {
                fo.println(genre.toString());
            }

        } catch ( FileNotFoundException ex ) {
            throw new SailoException("Tiedosto " + ftied.getName() + " ei aukea");
        } catch ( IOException ex ) {
            throw new SailoException("Tiedoston " + ftied.getName() + " kirjoittamisessa ongelmia");
        }

        muutettu = false;

    }
    
    
    /**
     * tarkistetaan taulun koko tällä hetkellä
     * @return taulukon koko
     */
    public int getLkm() {
        return alkiot.size();
    }
    
    
    /**
     * Asettaa tiedoston peruisnimen ilman tarkenninta
     * @param tied tallennustiedoston perusnimi
     */
    public void setTiedostonPerusNimi(String tied) {
        tiedostonPerusNimi = tied;
    }
    

    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */

    public String getTiedostonPerusNimi() {
        return tiedostonPerusNimi;
    }
    
    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonNimi() {
        return tiedostonPerusNimi + ".dat";
    }


    /**
     * Palauttaa varakopiotiedoston nimen
     * @return varakopiotiedoston nimi
     */
    public String getBakNimi() {
        return tiedostonPerusNimi + ".bak";
    }

      @Override
    public Iterator<PelinGenret> iterator() {
        return alkiot.iterator();
    }  
    

    
    
    /**
     * lukee pelien tiedostosta
     * ******KESKEN******
     *
     * @throws SailoException jos epäonnistuu
     */
    public static void lueTiedostosta() throws SailoException{
        tiedostonNimi = "peliengenret.dat";
    }
  
    
    /**
     * antaa pelille asetetut genret
     * @param tunnusnro etsittävä tunnusnumero pelille
     * @return pelin genret
     */
    public List<PelinGenret> annaPelienGenret(int tunnusnro) {
        List<PelinGenret> loydetyt = new ArrayList<PelinGenret>();
        for (PelinGenret genre : alkiot) {
            if (genre.getPeliId() == tunnusnro) loydetyt.add(genre);
        }
        return loydetyt;
    }
    
    /**
     * Pääohjelma
     * @param args ei oole
     */
    public static void main (String[] args) {
        //
    }
}
