package majuKomponentit;

import java.io.*;


import fi.jyu.mit.ohj2.*;


/**
 * PelinGenret joka osaa itse huolehtia omasta genreId:stä
 * 
 * @author jpaaj
 * @version 7.3.2025
 * 
 */
public class PelinGenret extends Genret {
    //Attribuutteja
    private int pelinGenretId; //Pelin Genret Id, koska pelillä voi olla useampi genre
    private int genreId; //Genren ID
    private int peliId; //Pelin ID
    private String genrenNimi; //Genre
    private int seuraavaNro = 1;
    
    
    
    /**
     * Alustetaan pelin genret, eipä vielä mittää
     */
    public PelinGenret() {
        pelinGenretId = rekisteroi();
        peliId = 0;
        genreId = 0;
    }


    /**
     * @param genreId genren ID
     * @param peliId pelin ID
     */
    public PelinGenret(int genreId, int peliId) {
        this.genreId = genreId;
        this.peliId = peliId;
        rekisteroi();
    }
    
    
    
    /**
     * Apumetodi, jolla saadaan täytettyä testiarvot Genrelle
     * @param nro viite peliin, jonka genrestä on kyse
     */
    public void vastaaTasoHyppely(int nro) {
        peliId = nro;
        genrenNimi = "Tasohyppely";
    }

    
    
    /**
     * Alustetaan tiety pelin genre(t)
     * @param genreId pelin viitenumero
     */
    public PelinGenret(int genreId) {
        this.genreId = genreId;
        rekisteroi();
    }
    
    
    /**
     * Tulostetaan Genren tiedot
     * @param out tietovirtaan tulostetaan
     */
    @Override
    public void tulosta(PrintStream out) {
        out.println("Pelin genre: " + peliId + " " + genreId);
    }
    
    
    
    /**
     * Tulostetaan pelin genren tiedot
     * @param os tietovirta johon tulostetaan
     */
    public void tulosta(OutputStream os) {
        tulosta(new PrintStream(os));
    }
    
    
    /**
     * Antaa PelinGenretId:lle rekisteri numeron, koska peleillä voi olla useampi genre
     * @return pelinGenretId:lle uusi numero
     */
    public int rekisteroi() {
        pelinGenretId = seuraavaNro;
        seuraavaNro++;
        return pelinGenretId;
    }
    
    
    /**
     * palautetaan genren oma id
     * @return genren id
     */
    public int getGenreId() {
        return genreId;
    }
    
    
    /**
     * @return palautta pelingenretID:n
     */
    public int getPelinGenretId() {
        return pelinGenretId;
    }
    
    /**
     * palautetaan mille pelille genre kuuluu
     * @return pelin id
     */
    public int getPeliId() {
        return peliId;
    }
    
    private void setPeliId(int nr) {
        peliId = nr;
        if ( peliId >= seuraavaNro ) seuraavaNro = peliId + 1;
    }
    
    private void setPelinGenretId(int nr1) {
        pelinGenretId = nr1;
        if ( pelinGenretId >= seuraavaNro ) seuraavaNro = pelinGenretId + 1;
    }

    
    /**
     * Palauttaa pelin genren tiedot merkkijonona jonka voi tallentaa tiedostoon.
     * @return genre tolppaeroteltuna merkkijonona 
     * @example
     * <pre name="test">
     *   Genre genre = new Genre();
     *   genre.parse("   1   |  7 |  Tasohyppely");
     *   genre.toString()    === "1|7|Tasohyppely";
     * </pre>
     */

    @Override
    public String toString() {
        return pelinGenretId + "|" + peliId + "|" + genreId;
    }

    
    /**
     * Selvitää Genren tiedot | erotellusta merkkijonosta.
     * Pitää huolen että seuraavaNro on suurempi kuin tuleva genreId.
     * @param rivi josta genren tiedot otetaan
     * @example
     * <pre name="test">
     *   Harrastus harrastus = new Harrastus();
     *   Genre genre = new Genre();
     *   genre.parse("   1   |  7 |  Tasohyppely");
     *   genre.toString()    === "1|7|Tasohyppely";
     *   
     *   genre.rekisteroi();
     *   int n = harrastus.getTunnusNro();
     *   harrastus.parse(""+(n+20));
     *   harrastus.rekisteroi();
     *   harrastus.getTunnusNro() === n+20+1;
     *   harrastus.toString()     === "" + (n+20+1) + "|10|Kalastus|1949|22";
     * </pre>
     */
    public void parse(String rivi) {
        StringBuffer sb = new StringBuffer(rivi);
        pelinGenretId = Mjonot.erota(sb,'|', pelinGenretId);
        peliId = Mjonot.erota(sb,'|', peliId);
        genreId = Mjonot.erota(sb, '|', genreId);
    }
    
    @Override
    public boolean equals(Object obj) {
        if ( obj == null ) return false;
        return this.toString().equals(obj.toString());
    }

    @Override
    public int hashCode() {
        return pelinGenretId;
    }

    
    /**
     * Pääohjelma
     * @param args no
     */
    public static void main (String[] args) {
        PelinGenret pelinGenret = new PelinGenret();
        pelinGenret.vastaaTasoHyppely(1);
        pelinGenret.tulosta(System.out);
    }
}
