package majuKomponentit;

import java.io.*;
import java.util.Arrays;

/**
 * Pelien tiedosto, joka voi lisätä uuden pelin
 * 
 * @author jpaaj
 * @version 7.3.2025
 *
 */
public class Pelit {
    //attribuutteja
    private static int    MAX_PELEJA      = 1;
    private static int                 lkm             = 0;
    private final String              tiedostonNimi   = "pelit.dat";
    private static Peli[] alkiot;
    private boolean muutettu = false;
    private String tiedostonPerusNimi = "nimet";

    /**
     * oletusmuodostaja
     */
    public Pelit() {
        alkiot = new Peli[MAX_PELEJA];
    }


    /**
     * Isomman taulun luonti
     * @param kasvatus kuinka paljon taulua kasvatetaan
     */
    public static void alkioKasvatus(int kasvatus) {
        alkiot = Arrays.copyOf(alkiot, MAX_PELEJA+kasvatus);
        MAX_PELEJA = alkiot.length;
    }


    /**
     * Pääohjelma
     * @param args e
     */
    public static void main(String[] args) {
        Pelit pelit = new Pelit();

        Peli vankilapako = new Peli("Vankilapako", "Majuri Studios", "Majuri Studios", 2024, "Windows", "Ei", "0/10", "10/10", "Kyllä");
        vankilapako.rekistoroi();

        Peli halfSword = new Peli("Half Sword", "Game Seer Publishing", "Half Sword Games", 2025, "Windows 10/11", "Ei", "0/10", "0/10", "Ei vielä");
        halfSword.rekistoroi();

        try {
            pelit.lisaa(vankilapako);
            pelit.lisaa(halfSword);
            for (int i = 0; i < pelit.getLkm(); i++) {
                Peli peli = pelit.anna(i);
                System.out.println("Pelin numero: " + (i+1));
                peli.tulosta(System.out);
            }
        } catch (SailoException e){
            System.out.println(e.getMessage());
        }
    }


    /**
     * Lisää uuden pelin tietorakenteeseen
     * @param peli lisättävän pelin viite
     * @throws SailoException jos tietorakenne on täynnä!
     */
    public void lisaa(Peli peli) throws SailoException {
        if (lkm >= alkiot.length) alkioKasvatus(5);//throw new SailoException("Liikaa Alkioita");
        alkiot[lkm++] = peli;
        muutettu = true;
    }
    

    /**
     * palauttaa viitteen i:teen peliin
     * @param i monennenko pelin viite halutaan
     * @return viite peliib, jonka indeksi on i
     * @throws IndexOutOfBoundsException jos i ei ole sallitulla alueella
     */
    public Peli anna(int i) throws IndexOutOfBoundsException{
        if (i < 0 || lkm <= i) 
            throw new IndexOutOfBoundsException("Laiton indeksi: " + i);
        return alkiot[i];
    }
    

    /**
     * Lukee pelin tiedostosta. ************KESSUN KESKEN*******************
     * @throws SailoException jos luku epäonnistuupi
     */
    public void lueTiedostosta() throws SailoException {
        try(BufferedReader fi = new BufferedReader(new FileReader(tiedostonNimi))) {
            String rivi = fi.readLine();
            if (rivi == null) throw new SailoException("Laiton indeksi: " + tiedostonNimi);
            while ((rivi = fi.readLine()) != null) {
                rivi = rivi.trim();
                if (rivi.isEmpty() || rivi.charAt(0) == ';') continue;
                Peli peli = new Peli();
                peli.parse(rivi);
                lisaa(peli);
            }
            muutettu = false;
        }catch (FileNotFoundException fe) {
            throw new SailoException("Tiedoston " + tiedostonNimi + " ei jostain syystä toimi: " + fe.getMessage());
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }
    

    /**
     * tallentaa pelin tiedostoon. ************KESSUN KESKEN*****************
     * @throws SailoException jos talletus epäonnistuu
     */
   public void tallenna() throws SailoException{
       if (!muutettu) return;

        try(PrintWriter ps = new PrintWriter(new FileOutputStream(tiedostonNimi))){
            ps.println(alkiot.length);
            for (Peli peli : alkiot) {
                ps.println(peli.toString());
            }
        } catch (FileNotFoundException fe) {
            throw new SailoException("Tiedostoa " + tiedostonNimi + " ei löydy!");
        }
        muutettu = false;
   }
    

    /**
     * Palauttaa pelien lukumäärän
     * @return pelien lukumäärä
     */
    public int getLkm() {
        return lkm;
    }
    
    //testiä
    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonPerusNimi() {
        return tiedostonPerusNimi;
    }


    /**
     * Asettaa tiedoston perusnimen ilan tarkenninta
     * @param nimi tallennustiedoston perusnimi
     */
    public void setTiedostonPerusNimi(String nimi) {
        tiedostonPerusNimi = nimi;
    }


    /**
     * Palauttaa tiedoston nimen, jota käytetään tallennukseen
     * @return tallennustiedoston nimi
     */
    public String getTiedostonNimi() {
        return getTiedostonPerusNimi() + ".dat";
    }


}

